// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GdkAnchorHints", sizeof(GdkAnchorHints), alignof(GdkAnchorHints));
    printf("%s;%zu;%zu\n", "GdkAxisFlags", sizeof(GdkAxisFlags), alignof(GdkAxisFlags));
    printf("%s;%zu;%zu\n", "GdkAxisUse", sizeof(GdkAxisUse), alignof(GdkAxisUse));
    printf("%s;%zu;%zu\n", "GdkByteOrder", sizeof(GdkByteOrder), alignof(GdkByteOrder));
    printf("%s;%zu;%zu\n", "GdkColor", sizeof(GdkColor), alignof(GdkColor));
    printf("%s;%zu;%zu\n", "GdkCrossingMode", sizeof(GdkCrossingMode), alignof(GdkCrossingMode));
    printf("%s;%zu;%zu\n", "GdkCursorType", sizeof(GdkCursorType), alignof(GdkCursorType));
    printf("%s;%zu;%zu\n", "GdkDevicePadFeature", sizeof(GdkDevicePadFeature), alignof(GdkDevicePadFeature));
    printf("%s;%zu;%zu\n", "GdkDeviceToolType", sizeof(GdkDeviceToolType), alignof(GdkDeviceToolType));
    printf("%s;%zu;%zu\n", "GdkDeviceType", sizeof(GdkDeviceType), alignof(GdkDeviceType));
    printf("%s;%zu;%zu\n", "GdkDragAction", sizeof(GdkDragAction), alignof(GdkDragAction));
    printf("%s;%zu;%zu\n", "GdkDragCancelReason", sizeof(GdkDragCancelReason), alignof(GdkDragCancelReason));
    printf("%s;%zu;%zu\n", "GdkDragProtocol", sizeof(GdkDragProtocol), alignof(GdkDragProtocol));
    printf("%s;%zu;%zu\n", "GdkEvent", sizeof(GdkEvent), alignof(GdkEvent));
    printf("%s;%zu;%zu\n", "GdkEventAny", sizeof(GdkEventAny), alignof(GdkEventAny));
    printf("%s;%zu;%zu\n", "GdkEventButton", sizeof(GdkEventButton), alignof(GdkEventButton));
    printf("%s;%zu;%zu\n", "GdkEventConfigure", sizeof(GdkEventConfigure), alignof(GdkEventConfigure));
    printf("%s;%zu;%zu\n", "GdkEventCrossing", sizeof(GdkEventCrossing), alignof(GdkEventCrossing));
    printf("%s;%zu;%zu\n", "GdkEventDND", sizeof(GdkEventDND), alignof(GdkEventDND));
    printf("%s;%zu;%zu\n", "GdkEventExpose", sizeof(GdkEventExpose), alignof(GdkEventExpose));
    printf("%s;%zu;%zu\n", "GdkEventFocus", sizeof(GdkEventFocus), alignof(GdkEventFocus));
    printf("%s;%zu;%zu\n", "GdkEventGrabBroken", sizeof(GdkEventGrabBroken), alignof(GdkEventGrabBroken));
    printf("%s;%zu;%zu\n", "GdkEventKey", sizeof(GdkEventKey), alignof(GdkEventKey));
    printf("%s;%zu;%zu\n", "GdkEventMask", sizeof(GdkEventMask), alignof(GdkEventMask));
    printf("%s;%zu;%zu\n", "GdkEventMotion", sizeof(GdkEventMotion), alignof(GdkEventMotion));
    printf("%s;%zu;%zu\n", "GdkEventOwnerChange", sizeof(GdkEventOwnerChange), alignof(GdkEventOwnerChange));
    printf("%s;%zu;%zu\n", "GdkEventPadAxis", sizeof(GdkEventPadAxis), alignof(GdkEventPadAxis));
    printf("%s;%zu;%zu\n", "GdkEventPadButton", sizeof(GdkEventPadButton), alignof(GdkEventPadButton));
    printf("%s;%zu;%zu\n", "GdkEventPadGroupMode", sizeof(GdkEventPadGroupMode), alignof(GdkEventPadGroupMode));
    printf("%s;%zu;%zu\n", "GdkEventProperty", sizeof(GdkEventProperty), alignof(GdkEventProperty));
    printf("%s;%zu;%zu\n", "GdkEventProximity", sizeof(GdkEventProximity), alignof(GdkEventProximity));
    printf("%s;%zu;%zu\n", "GdkEventScroll", sizeof(GdkEventScroll), alignof(GdkEventScroll));
    printf("%s;%zu;%zu\n", "GdkEventSelection", sizeof(GdkEventSelection), alignof(GdkEventSelection));
    printf("%s;%zu;%zu\n", "GdkEventSetting", sizeof(GdkEventSetting), alignof(GdkEventSetting));
    printf("%s;%zu;%zu\n", "GdkEventTouch", sizeof(GdkEventTouch), alignof(GdkEventTouch));
    printf("%s;%zu;%zu\n", "GdkEventTouchpadPinch", sizeof(GdkEventTouchpadPinch), alignof(GdkEventTouchpadPinch));
    printf("%s;%zu;%zu\n", "GdkEventTouchpadSwipe", sizeof(GdkEventTouchpadSwipe), alignof(GdkEventTouchpadSwipe));
    printf("%s;%zu;%zu\n", "GdkEventType", sizeof(GdkEventType), alignof(GdkEventType));
    printf("%s;%zu;%zu\n", "GdkEventVisibility", sizeof(GdkEventVisibility), alignof(GdkEventVisibility));
    printf("%s;%zu;%zu\n", "GdkEventWindowState", sizeof(GdkEventWindowState), alignof(GdkEventWindowState));
    printf("%s;%zu;%zu\n", "GdkFilterReturn", sizeof(GdkFilterReturn), alignof(GdkFilterReturn));
    printf("%s;%zu;%zu\n", "GdkFrameClockPhase", sizeof(GdkFrameClockPhase), alignof(GdkFrameClockPhase));
    printf("%s;%zu;%zu\n", "GdkFullscreenMode", sizeof(GdkFullscreenMode), alignof(GdkFullscreenMode));
    printf("%s;%zu;%zu\n", "GdkGLError", sizeof(GdkGLError), alignof(GdkGLError));
    printf("%s;%zu;%zu\n", "GdkGeometry", sizeof(GdkGeometry), alignof(GdkGeometry));
    printf("%s;%zu;%zu\n", "GdkGrabOwnership", sizeof(GdkGrabOwnership), alignof(GdkGrabOwnership));
    printf("%s;%zu;%zu\n", "GdkGrabStatus", sizeof(GdkGrabStatus), alignof(GdkGrabStatus));
    printf("%s;%zu;%zu\n", "GdkGravity", sizeof(GdkGravity), alignof(GdkGravity));
    printf("%s;%zu;%zu\n", "GdkInputMode", sizeof(GdkInputMode), alignof(GdkInputMode));
    printf("%s;%zu;%zu\n", "GdkInputSource", sizeof(GdkInputSource), alignof(GdkInputSource));
    printf("%s;%zu;%zu\n", "GdkKeymapKey", sizeof(GdkKeymapKey), alignof(GdkKeymapKey));
    printf("%s;%zu;%zu\n", "GdkModifierIntent", sizeof(GdkModifierIntent), alignof(GdkModifierIntent));
    printf("%s;%zu;%zu\n", "GdkModifierType", sizeof(GdkModifierType), alignof(GdkModifierType));
    printf("%s;%zu;%zu\n", "GdkNotifyType", sizeof(GdkNotifyType), alignof(GdkNotifyType));
    printf("%s;%zu;%zu\n", "GdkOwnerChange", sizeof(GdkOwnerChange), alignof(GdkOwnerChange));
    printf("%s;%zu;%zu\n", "GdkPoint", sizeof(GdkPoint), alignof(GdkPoint));
    printf("%s;%zu;%zu\n", "GdkPropMode", sizeof(GdkPropMode), alignof(GdkPropMode));
    printf("%s;%zu;%zu\n", "GdkPropertyState", sizeof(GdkPropertyState), alignof(GdkPropertyState));
    printf("%s;%zu;%zu\n", "GdkRGBA", sizeof(GdkRGBA), alignof(GdkRGBA));
    printf("%s;%zu;%zu\n", "GdkRectangle", sizeof(GdkRectangle), alignof(GdkRectangle));
    printf("%s;%zu;%zu\n", "GdkScrollDirection", sizeof(GdkScrollDirection), alignof(GdkScrollDirection));
    printf("%s;%zu;%zu\n", "GdkSeat", sizeof(GdkSeat), alignof(GdkSeat));
    printf("%s;%zu;%zu\n", "GdkSeatCapabilities", sizeof(GdkSeatCapabilities), alignof(GdkSeatCapabilities));
    printf("%s;%zu;%zu\n", "GdkSettingAction", sizeof(GdkSettingAction), alignof(GdkSettingAction));
    printf("%s;%zu;%zu\n", "GdkStatus", sizeof(GdkStatus), alignof(GdkStatus));
    printf("%s;%zu;%zu\n", "GdkSubpixelLayout", sizeof(GdkSubpixelLayout), alignof(GdkSubpixelLayout));
    printf("%s;%zu;%zu\n", "GdkTimeCoord", sizeof(GdkTimeCoord), alignof(GdkTimeCoord));
    printf("%s;%zu;%zu\n", "GdkTouchpadGesturePhase", sizeof(GdkTouchpadGesturePhase), alignof(GdkTouchpadGesturePhase));
    printf("%s;%zu;%zu\n", "GdkVisibilityState", sizeof(GdkVisibilityState), alignof(GdkVisibilityState));
    printf("%s;%zu;%zu\n", "GdkVisualType", sizeof(GdkVisualType), alignof(GdkVisualType));
    printf("%s;%zu;%zu\n", "GdkWMDecoration", sizeof(GdkWMDecoration), alignof(GdkWMDecoration));
    printf("%s;%zu;%zu\n", "GdkWMFunction", sizeof(GdkWMFunction), alignof(GdkWMFunction));
    printf("%s;%zu;%zu\n", "GdkWindowAttr", sizeof(GdkWindowAttr), alignof(GdkWindowAttr));
    printf("%s;%zu;%zu\n", "GdkWindowAttributesType", sizeof(GdkWindowAttributesType), alignof(GdkWindowAttributesType));
    printf("%s;%zu;%zu\n", "GdkWindowClass", sizeof(GdkWindowClass), alignof(GdkWindowClass));
    printf("%s;%zu;%zu\n", "GdkWindowEdge", sizeof(GdkWindowEdge), alignof(GdkWindowEdge));
    printf("%s;%zu;%zu\n", "GdkWindowHints", sizeof(GdkWindowHints), alignof(GdkWindowHints));
    printf("%s;%zu;%zu\n", "GdkWindowState", sizeof(GdkWindowState), alignof(GdkWindowState));
    printf("%s;%zu;%zu\n", "GdkWindowType", sizeof(GdkWindowType), alignof(GdkWindowType));
    printf("%s;%zu;%zu\n", "GdkWindowTypeHint", sizeof(GdkWindowTypeHint), alignof(GdkWindowTypeHint));
    printf("%s;%zu;%zu\n", "GdkWindowWindowClass", sizeof(GdkWindowWindowClass), alignof(GdkWindowWindowClass));
    return 0;
}
