// Take a look at the license at the top of the repository in the LICENSE file.

//! Traits intended for blanket imports.

pub use crate::auto::traits::*;
pub use crate::editable_text::EditableTextExtManual;
pub use crate::table::TableExtManual;
#[doc(hidden)]
pub use glib::prelude::*;
